@@ -8,6 +8,7 @@ import (
 	"github.com/stretchr/testify/require"
 
 	"github.com/influxdata/telegraf"
+	telegrafConfig "github.com/influxdata/telegraf/config"
 	"github.com/influxdata/telegraf/metric"
 	"github.com/influxdata/telegraf/testutil"
 )
@@ -16,11 +17,23 @@ type fields map[string]interface{}
 type tags map[string]string
 
 // NewTestHistogram creates new test histogram aggregation with specified config
-func NewTestHistogram(cfg []config, reset bool, cumulative bool) telegraf.Aggregator {
+func NewTestHistogram(cfg []config, reset bool, cumulative bool, pushOnlyOnUpdate bool) telegraf.Aggregator {
+	return NewTestHistogramWithExpirationInterval(cfg, reset, cumulative, pushOnlyOnUpdate, 0)
+}
+
+func NewTestHistogramWithExpirationInterval(
+	cfg []config,
+	reset bool,
+	cumulative bool,
+	pushOnlyOnUpdate bool,
+	expirationInterval telegrafConfig.Duration,
+) telegraf.Aggregator {
 	htm := NewHistogramAggregator()
 	htm.Configs = cfg
 	htm.ResetBuckets = reset
 	htm.Cumulative = cumulative
+	htm.ExpirationInterval = expirationInterval
+	htm.PushOnlyOnUpdate = pushOnlyOnUpdate
 
 	return htm
 }
@@ -74,7 +87,7 @@ func BenchmarkApply(b *testing.B) {
 func TestHistogram(t *testing.T) {
 	var cfg []config
 	cfg = append(cfg, config{Metric: "first_metric_name", Fields: []string{"a"}, Buckets: []float64{0.0, 10.0, 20.0, 30.0, 40.0}})
-	histogram := NewTestHistogram(cfg, false, true)
+	histogram := NewTestHistogram(cfg, false, true, false)
 
 	acc := &testutil.Accumulator{}
 
@@ -92,11 +105,47 @@ func TestHistogram(t *testing.T) {
 	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2)}, tags{bucketRightTag: bucketPosInf})
 }
 
+// TestHistogram tests metrics for one period, for one field and push only on histogram update
+func TestHistogramPushOnUpdate(t *testing.T) {
+	var cfg []config
+	cfg = append(cfg, config{Metric: "first_metric_name", Fields: []string{"a"}, Buckets: []float64{0.0, 10.0, 20.0, 30.0, 40.0}})
+	histogram := NewTestHistogram(cfg, false, true, true)
+
+	acc := &testutil.Accumulator{}
+
+	histogram.Add(firstMetric1)
+	histogram.Reset()
+	histogram.Add(firstMetric2)
+	histogram.Push(acc)
+
+	require.Len(t, acc.Metrics, 6, "Incorrect number of metrics")
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(0)}, tags{bucketRightTag: "0"})
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(0)}, tags{bucketRightTag: "10"})
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2)}, tags{bucketRightTag: "20"})
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2)}, tags{bucketRightTag: "30"})
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2)}, tags{bucketRightTag: "40"})
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2)}, tags{bucketRightTag: bucketPosInf})
+
+	acc.ClearMetrics()
+	histogram.Push(acc)
+	require.Len(t, acc.Metrics, 0, "Incorrect number of metrics")
+	histogram.Add(firstMetric2)
+	histogram.Push(acc)
+
+	require.Len(t, acc.Metrics, 6, "Incorrect number of metrics")
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(0)}, tags{bucketRightTag: "0"})
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(0)}, tags{bucketRightTag: "10"})
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(3)}, tags{bucketRightTag: "20"})
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(3)}, tags{bucketRightTag: "30"})
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(3)}, tags{bucketRightTag: "40"})
+	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(3)}, tags{bucketRightTag: bucketPosInf})
+}
+
 // TestHistogramNonCumulative tests metrics for one period and for one field
 func TestHistogramNonCumulative(t *testing.T) {
 	var cfg []config
 	cfg = append(cfg, config{Metric: "first_metric_name", Fields: []string{"a"}, Buckets: []float64{0.0, 10.0, 20.0, 30.0, 40.0}})
-	histogram := NewTestHistogram(cfg, false, false)
+	histogram := NewTestHistogram(cfg, false, false, false)
 
 	acc := &testutil.Accumulator{}
 
@@ -118,7 +167,7 @@ func TestHistogramNonCumulative(t *testing.T) {
 func TestHistogramWithReset(t *testing.T) {
 	var cfg []config
 	cfg = append(cfg, config{Metric: "first_metric_name", Fields: []string{"a"}, Buckets: []float64{0.0, 10.0, 20.0, 30.0, 40.0}})
-	histogram := NewTestHistogram(cfg, true, true)
+	histogram := NewTestHistogram(cfg, true, true, false)
 
 	acc := &testutil.Accumulator{}
 
@@ -141,7 +190,7 @@ func TestHistogramWithAllFields(t *testing.T) {
 	var cfg []config
 	cfg = append(cfg, config{Metric: "first_metric_name", Buckets: []float64{0.0, 15.5, 20.0, 30.0, 40.0}})
 	cfg = append(cfg, config{Metric: "second_metric_name", Buckets: []float64{0.0, 4.0, 10.0, 23.0, 30.0}})
-	histogram := NewTestHistogram(cfg, false, true)
+	histogram := NewTestHistogram(cfg, false, true, false)
 
 	acc := &testutil.Accumulator{}
 
@@ -152,18 +201,66 @@ func TestHistogramWithAllFields(t *testing.T) {
 
 	require.Len(t, acc.Metrics, 12, "Incorrect number of metrics")
 	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(0), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketRightTag: "0"})
-	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(1), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketRightTag: "15.5"})
+	assertContainsTaggedField(
+		t,
+		acc,
+		"first_metric_name",
+		fields{"a_bucket": int64(1), "b_bucket": int64(0), "c_bucket": int64(0)},
+		tags{bucketRightTag: "15.5"},
+	)
 	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketRightTag: "20"})
 	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketRightTag: "30"})
 	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2), "b_bucket": int64(1), "c_bucket": int64(1)}, tags{bucketRightTag: "40"})
-	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2), "b_bucket": int64(1), "c_bucket": int64(1)}, tags{bucketRightTag: bucketPosInf})
+	assertContainsTaggedField(
+		t,
+		acc,
+		"first_metric_name",
+		fields{"a_bucket": int64(2), "b_bucket": int64(1), "c_bucket": int64(1)},
+		tags{bucketRightTag: bucketPosInf},
+	)
 
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketRightTag: "0"})
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketRightTag: "4"})
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketRightTag: "10"})
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketRightTag: "23"})
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketRightTag: "30"})
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(1), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketRightTag: bucketPosInf})
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: "0"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: "4"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: "10"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: "23"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: "30"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(1), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: bucketPosInf},
+	)
 }
 
 // TestHistogramWithAllFieldsNonCumulative tests two metrics for one period and for all fields
@@ -171,7 +268,7 @@ func TestHistogramWithAllFieldsNonCumulative(t *testing.T) {
 	var cfg []config
 	cfg = append(cfg, config{Metric: "first_metric_name", Buckets: []float64{0.0, 15.5, 20.0, 30.0, 40.0}})
 	cfg = append(cfg, config{Metric: "second_metric_name", Buckets: []float64{0.0, 4.0, 10.0, 23.0, 30.0}})
-	histogram := NewTestHistogram(cfg, false, false)
+	histogram := NewTestHistogram(cfg, false, false, false)
 
 	acc := &testutil.Accumulator{}
 
@@ -181,19 +278,91 @@ func TestHistogramWithAllFieldsNonCumulative(t *testing.T) {
 	histogram.Push(acc)
 
 	require.Len(t, acc.Metrics, 12, "Incorrect number of metrics")
-	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(0), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketLeftTag: bucketNegInf, bucketRightTag: "0"})
-	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(1), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketLeftTag: "0", bucketRightTag: "15.5"})
-	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(1), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketLeftTag: "15.5", bucketRightTag: "20"})
-	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(0), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketLeftTag: "20", bucketRightTag: "30"})
-	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(0), "b_bucket": int64(1), "c_bucket": int64(1)}, tags{bucketLeftTag: "30", bucketRightTag: "40"})
-	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(0), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketLeftTag: "40", bucketRightTag: bucketPosInf})
+	assertContainsTaggedField(
+		t,
+		acc,
+		"first_metric_name",
+		fields{"a_bucket": int64(0), "b_bucket": int64(0), "c_bucket": int64(0)},
+		tags{bucketLeftTag: bucketNegInf, bucketRightTag: "0"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"first_metric_name",
+		fields{"a_bucket": int64(1), "b_bucket": int64(0), "c_bucket": int64(0)},
+		tags{bucketLeftTag: "0", bucketRightTag: "15.5"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"first_metric_name",
+		fields{"a_bucket": int64(1), "b_bucket": int64(0), "c_bucket": int64(0)},
+		tags{bucketLeftTag: "15.5", bucketRightTag: "20"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"first_metric_name",
+		fields{"a_bucket": int64(0), "b_bucket": int64(0), "c_bucket": int64(0)},
+		tags{bucketLeftTag: "20", bucketRightTag: "30"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"first_metric_name",
+		fields{"a_bucket": int64(0), "b_bucket": int64(1), "c_bucket": int64(1)},
+		tags{bucketLeftTag: "30", bucketRightTag: "40"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"first_metric_name",
+		fields{"a_bucket": int64(0), "b_bucket": int64(0), "c_bucket": int64(0)},
+		tags{bucketLeftTag: "40", bucketRightTag: bucketPosInf},
+	)
 
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketLeftTag: bucketNegInf, bucketRightTag: "0"})
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketLeftTag: "0", bucketRightTag: "4"})
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketLeftTag: "4", bucketRightTag: "10"})
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketLeftTag: "10", bucketRightTag: "23"})
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketLeftTag: "23", bucketRightTag: "30"})
-	assertContainsTaggedField(t, acc, "second_metric_name", fields{"a_bucket": int64(1), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)}, tags{bucketLeftTag: "30", bucketRightTag: bucketPosInf})
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketLeftTag: bucketNegInf, bucketRightTag: "0"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketLeftTag: "0", bucketRightTag: "4"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketLeftTag: "4", bucketRightTag: "10"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketLeftTag: "10", bucketRightTag: "23"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketLeftTag: "23", bucketRightTag: "30"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(1), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketLeftTag: "30", bucketRightTag: bucketPosInf},
+	)
 }
 
 // TestHistogramWithTwoPeriodsAndAllFields tests two metrics getting added with a push/reset in between (simulates
@@ -201,7 +370,7 @@ func TestHistogramWithAllFieldsNonCumulative(t *testing.T) {
 func TestHistogramWithTwoPeriodsAndAllFields(t *testing.T) {
 	var cfg []config
 	cfg = append(cfg, config{Metric: "first_metric_name", Buckets: []float64{0.0, 10.0, 20.0, 30.0, 40.0}})
-	histogram := NewTestHistogram(cfg, false, true)
+	histogram := NewTestHistogram(cfg, false, true, false)
 
 	acc := &testutil.Accumulator{}
 	histogram.Add(firstMetric1)
@@ -223,7 +392,13 @@ func TestHistogramWithTwoPeriodsAndAllFields(t *testing.T) {
 	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketRightTag: "20"})
 	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2), "b_bucket": int64(0), "c_bucket": int64(0)}, tags{bucketRightTag: "30"})
 	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2), "b_bucket": int64(1), "c_bucket": int64(1)}, tags{bucketRightTag: "40"})
-	assertContainsTaggedField(t, acc, "first_metric_name", fields{"a_bucket": int64(2), "b_bucket": int64(1), "c_bucket": int64(1)}, tags{bucketRightTag: bucketPosInf})
+	assertContainsTaggedField(
+		t,
+		acc,
+		"first_metric_name",
+		fields{"a_bucket": int64(2), "b_bucket": int64(1), "c_bucket": int64(1)},
+		tags{bucketRightTag: bucketPosInf},
+	)
 }
 
 // TestWrongBucketsOrder tests the calling panic with incorrect order of buckets
@@ -240,10 +415,77 @@ func TestWrongBucketsOrder(t *testing.T) {
 
 	var cfg []config
 	cfg = append(cfg, config{Metric: "first_metric_name", Buckets: []float64{0.0, 90.0, 20.0, 30.0, 40.0}})
-	histogram := NewTestHistogram(cfg, false, true)
+	histogram := NewTestHistogram(cfg, false, true, false)
 	histogram.Add(firstMetric2)
 }
 
+// TestHistogram tests two metrics getting added and metric expiration
+func TestHistogramMetricExpiration(t *testing.T) {
+	currentTime := time.Unix(10, 0)
+	timeNow = func() time.Time {
+		return currentTime
+	}
+	defer func() {
+		timeNow = time.Now
+	}()
+
+	var cfg []config
+	cfg = append(cfg, config{Metric: "first_metric_name", Fields: []string{"a"}, Buckets: []float64{0.0, 10.0, 20.0, 30.0, 40.0}})
+	cfg = append(cfg, config{Metric: "second_metric_name", Buckets: []float64{0.0, 4.0, 10.0, 23.0, 30.0}})
+	histogram := NewTestHistogramWithExpirationInterval(cfg, false, true, false, telegrafConfig.Duration(30))
+
+	acc := &testutil.Accumulator{}
+
+	histogram.Add(firstMetric1)
+	currentTime = time.Unix(41, 0)
+	histogram.Add(secondMetric)
+	histogram.Push(acc)
+
+	require.Len(t, acc.Metrics, 6, "Incorrect number of metrics")
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: "0"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: "4"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: "10"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: "23"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(0), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: "30"},
+	)
+	assertContainsTaggedField(
+		t,
+		acc,
+		"second_metric_name",
+		fields{"a_bucket": int64(1), "ignoreme_bucket": int64(0), "andme_bucket": int64(0)},
+		tags{bucketRightTag: bucketPosInf},
+	)
+}
+
 // assertContainsTaggedField is help functions to test histogram data
 func assertContainsTaggedField(t *testing.T, acc *testutil.Accumulator, metricName string, fields map[string]interface{}, tags map[string]string) {
 	acc.Lock()
